@@ -36,13 +36,15 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <AvailabilityMacros.h>
 #endif
 
-#ifdef __GNUC__
-
-#include <cxxabi.h>
 #include <string>
 #include <cstring>
 #include <stdlib.h>
 
+// uClibc++ doesn't have cxxabi.h
+#if defined __GNUC__ && !defined __UCLIBCXX_MAJOR__
+
+#include <cxxabi.h>
+
 std::string demangle(char const* name)
 {
 // in case this string comes
@@ -81,6 +83,8 @@ std::string demangle(char const* name)
 	return ret;
 }
 
+#else
+std::string demangle(char const* name) { return name; }
 #endif
 
 #include <stdlib.h>
