@@ -1230,7 +1230,7 @@ namespace libtorrent
 			(*m_logger) << time_now_string() << " <== HASHPIECE " << p.piece << " list: " << list_size << " ";
 #endif
 			lazy_entry hash_list;
-			if (lazy_bdecode(recv_buffer.begin + 13, recv_buffer.end + 13 + list_size, hash_list) != 0)
+			if (lazy_bdecode(recv_buffer.begin + 13, recv_buffer.begin + 13 + list_size, hash_list) != 0)
 			{
 				disconnect(errors::invalid_hash_piece, 2);
 				return;
@@ -1465,7 +1465,7 @@ namespace libtorrent
 		if (extended_id == upload_only_msg)
 		{
 			if (!packet_finished()) return;
-			if (packet_size() != 1) return;
+			if (packet_size() != 3) return;
 			set_upload_only(detail::read_uint8(recv_buffer.begin));
 			return;
 		}
@@ -1473,7 +1473,7 @@ namespace libtorrent
 		if (extended_id == dont_have_msg)
 		{
 			if (!packet_finished()) return;
-			if (packet_size() != 4) return;
+			if (packet_size() != 6) return;
 			int piece = detail::read_uint32(recv_buffer.begin) != 0;
 			incoming_dont_have(piece);
 			return;
