@@ -347,6 +347,11 @@ namespace libtorrent
 				}
 				break;
 			}
+			case lazy_entry::none_t:
+			{
+				destruct();
+				break;
+			}
 		}
 	}
 
