@@ -172,19 +172,19 @@ namespace libtorrent
 		const static open_mode_t mode_array[] =
 		{
 			// read_only
-			{GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS},
+			{GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, 0},
 			// write_only
-			{GENERIC_WRITE, FILE_SHARE_READ, OPEN_ALWAYS, FILE_FLAG_RANDOM_ACCESS},
+			{GENERIC_WRITE, FILE_SHARE_READ, OPEN_ALWAYS, 0},
 			// read_write
-			{GENERIC_WRITE | GENERIC_READ, FILE_SHARE_READ, OPEN_ALWAYS, FILE_FLAG_RANDOM_ACCESS},
+			{GENERIC_WRITE | GENERIC_READ, FILE_SHARE_READ, OPEN_ALWAYS, 0},
 			// invalid option
 			{0,0,0,0},
 			// read_only no_buffer
-			{GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS | FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING },
+			{GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING },
 			// write_only no_buffer
-			{GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_ALWAYS, FILE_FLAG_RANDOM_ACCESS | FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING },
+			{GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_ALWAYS, FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING },
 			// read_write no_buffer
-			{GENERIC_WRITE | GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_ALWAYS, FILE_FLAG_RANDOM_ACCESS | FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING },
+			{GENERIC_WRITE | GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_ALWAYS, FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING },
 			// invalid option
 			{0,0,0,0}
 		};
@@ -209,8 +209,11 @@ namespace libtorrent
 		open_mode_t const& m = mode_array[mode & mode_mask];
 		DWORD a = attrib_array[(mode & attribute_mask) >> 12];
 
+		DWORD extra_flags = ((mode & random_access) ? FILE_FLAG_RANDOM_ACCESS : 0)
+			| (a ? a : FILE_ATTRIBUTE_NORMAL);
+
 		m_file_handle = CreateFile_(m_path.c_str(), m.rw_mode, m.share_mode, 0
-			, m.create_mode, m.flags | (a ? a : FILE_ATTRIBUTE_NORMAL), 0);
+			, m.create_mode, m.flags | extra_flags, 0);
 
 		if (m_file_handle == INVALID_HANDLE_VALUE)
 		{
@@ -272,8 +275,11 @@ namespace libtorrent
 #endif
 
 #ifdef POSIX_FADV_RANDOM
-		// disable read-ahead
-		posix_fadvise(m_fd, 0, 0, POSIX_FADV_RANDOM);
+		if (mode & random_access)
+		{
+			// disable read-ahead
+			posix_fadvise(m_fd, 0, 0, POSIX_FADV_RANDOM);
+		}
 #endif
 
 #endif
@@ -503,7 +509,8 @@ namespace libtorrent
 				CloseHandle(ol.hEvent);
 				return -1;
 			}
-			if (GetOverlappedResult(m_file_handle, &ol, &ret, true) == 0)
+			DWORD num_read;
+			if (GetOverlappedResult(m_file_handle, &ol, &num_read, true) == 0)
 			{
 				if (GetLastError() != ERROR_HANDLE_EOF)
 				{
@@ -512,6 +519,7 @@ namespace libtorrent
 					return -1;
 				}
 			}
+			if (num_read < ret) ret = num_read;
 		}
 		CloseHandle(ol.hEvent);
 		return ret;
@@ -627,7 +635,16 @@ namespace libtorrent
 				size += i->iov_len;
 			}
 			error_code code;
-			if (eof) TORRENT_ASSERT(file_offset + size >= get_size(code));
+			if (eof) 
+			{
+				size_type fsize = get_size(code);
+				if (code) printf("get_size: %s\n", code.message().c_str());
+				if (file_offset + size < fsize)
+				{
+					printf("offset: %d size: %d get_size: %d\n", int(file_offset), int(size), int(fsize));
+					TORRENT_ASSERT(false);
+				}
+			}
 		}
 #endif
 
@@ -694,17 +711,12 @@ namespace libtorrent
 		ol.hEvent = CreateEvent(0, true, false, 0);
 
 		ret += size;
-		// if file_size is > 0, the file will be opened in unbuffered
-		// mode after the write completes, and truncate the file to
-		// file_size.
 		size_type file_size = 0;
 	
 		if ((size & (m_page_size-1)) != 0)
 		{
 			// if size is not an even multiple, this must be the tail
-			// of the file. Write the whole page and then open a new
-			// file without FILE_FLAG_NO_BUFFERING and set the
-			// file size to file_offset + size
+			// of the file.
 
 			file_size = file_offset + size;
 			size = num_pages * m_page_size;
@@ -719,51 +731,18 @@ namespace libtorrent
 				CloseHandle(ol.hEvent);
 				return -1;
 			}
-			DWORD tmp;
-			if (GetOverlappedResult(m_file_handle, &ol, &tmp, true) == 0)
+			DWORD num_written;
+			if (GetOverlappedResult(m_file_handle, &ol, &num_written, true) == 0)
 			{
 				ec = error_code(GetLastError(), get_system_category());
 				CloseHandle(ol.hEvent);
 				return -1;
 			}
-			if (tmp < ret) ret = tmp;
+			if (num_written < ret) ret = num_written;
 		}
 		CloseHandle(ol.hEvent);
 
-		if (file_size > 0)
-		{
-#if TORRENT_USE_WPATH
-#define CreateFile_ CreateFileW
-#else
-#define CreateFile_ CreateFileA
-#endif
-			HANDLE f = CreateFile_(m_path.c_str(), GENERIC_WRITE
-			, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING
-			, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, 0);
-
-			if (f == INVALID_HANDLE_VALUE)
-			{
-				ec = error_code(GetLastError(), get_system_category());
-				return -1;
-			}
-
-			LARGE_INTEGER offs;
-			offs.QuadPart = file_size;
-			if (SetFilePointerEx(f, offs, &offs, FILE_BEGIN) == FALSE)
-			{
-				CloseHandle(f);
-				ec = error_code(GetLastError(), get_system_category());
-				return -1;
-			}
-			if (::SetEndOfFile(f) == FALSE)
-			{
-				ec = error_code(GetLastError(), get_system_category());
-				CloseHandle(f);
-				return -1;
-			}
-			CloseHandle(f);
-		}
-
+		if (file_size > 0) set_size(file_size, ec);
 		return ret;
 #else
 		size_type ret = lseek(m_fd, file_offset, SEEK_SET);
@@ -928,6 +907,53 @@ namespace libtorrent
   		TORRENT_ASSERT(s >= 0);
 
 #ifdef TORRENT_WINDOWS
+
+		if ((m_open_mode & no_buffer) && (s & (size_alignment()-1)) != 0)
+		{
+			// the file is opened in unbuffered mode, and the size is not
+			// aligned to the required cluster size. Use NtSetInformationFile
+
+#define FileEndOfFileInformation 20
+#ifndef NT_SUCCESS
+#define NT_SUCCESS(x) (!((x) & 0x80000000))
+#endif
+			
+			// for NtSetInformationFile, see: 
+			// http://undocumented.ntinternals.net/UserMode/Undocumented%20Functions/NT%20Objects/File/NtSetInformationFile.html
+
+			typedef DWORD _NTSTATUS;
+			typedef _NTSTATUS (NTAPI * NtSetInformationFile_t)(HANDLE file, PULONG_PTR iosb, PVOID data, ULONG len, ULONG file_info_class);
+
+			static NtSetInformationFile_t NtSetInformationFile = 0;
+			static bool failed_ntdll = false;
+
+			if (NtSetInformationFile == 0 && !failed_ntdll)
+			{
+				HMODULE nt = LoadLibraryA("ntdll");
+				if (nt)
+				{
+					NtSetInformationFile = (NtSetInformationFile_t)GetProcAddress(nt, "NtSetInformationFile");
+					if (NtSetInformationFile == 0) failed_ntdll = true;
+				}
+				else failed_ntdll = true;
+			}
+
+			if (!failed_ntdll && NtSetInformationFile)
+			{
+				ULONG_PTR Iosb[2];
+				LARGE_INTEGER fsize;
+				fsize.QuadPart = s;
+				_NTSTATUS st = NtSetInformationFile(m_file_handle
+					, Iosb, &fsize, sizeof(fsize), FileEndOfFileInformation);
+				if (!NT_SUCCESS(st)) 
+				{
+					ec.assign(INVALID_SET_FILE_POINTER, get_system_category());
+					return false;
+				}
+				return true;
+			}
+		}
+
 		LARGE_INTEGER offs;
 		LARGE_INTEGER cur_size;
 		if (GetFileSizeEx(m_file_handle, &cur_size) == FALSE)
