@@ -57,18 +57,15 @@ namespace libtorrent
 				s.get<http_stream>()->set_username(ps.username, ps.password);
 		}
 		else if (ps.type == proxy_settings::socks5
-			|| ps.type == proxy_settings::socks5_pw)
+			|| ps.type == proxy_settings::socks5_pw
+			|| ps.type == proxy_settings::socks4)
 		{
 			s.instantiate<socks5_stream>(ios);
 			s.get<socks5_stream>()->set_proxy(ps.hostname, ps.port);
 			if (ps.type == proxy_settings::socks5_pw)
 				s.get<socks5_stream>()->set_username(ps.username, ps.password);
-		}
-		else if (ps.type == proxy_settings::socks4)
-		{
-			s.instantiate<socks4_stream>(ios);
-			s.get<socks4_stream>()->set_proxy(ps.hostname, ps.port);
-			s.get<socks4_stream>()->set_username(ps.username);
+			if (ps.type == proxy_settings::socks4)
+				s.get<socks5_stream>()->set_version(4);
 		}
 		else
 		{
